package com.landray.kmss.sys.attachment.integrate.wps.util;

import java.util.Collection;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.hwpf.usermodel.Bookmark;
import org.apache.poi.hwpf.usermodel.Bookmarks;
import org.apache.commons.io.IOUtils;
import org.apache.poi.hwpf.HWPFDocument;
import java.util.Iterator;
import com.landray.kmss.util.ArrayUtil;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.io.ByteArrayOutputStream;
import java.util.List;
import java.io.InputStream;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

public class BookMarkUtil
{
    private static final Logger logger;
    
    static {
        logger = LoggerFactory.getLogger(BookMarkUtil.class);
    }
    
    public static List<String> getBookMarkList(final InputStream in, final String ext) throws Exception {
        if (in == null) {
            return null;
        }
        final byte[] buffer = new byte[1024];
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        int len;
        while ((len = in.read(buffer)) > -1) {
            baos.write(buffer, 0, len);
        }
        baos.flush();
        if (in != null) {
            in.close();
        }
        if (baos != null) {
            baos.close();
        }
        final byte[] resultBuffer = baos.toByteArray();
        List<String> bookList = new ArrayList<String>();
        if ("doc".equals(ext)) {
            try {
                bookList = getDocBookMark(new ByteArrayInputStream(resultBuffer));
            }
            catch (final Exception ex) {
                bookList = getDocxBookMark(new ByteArrayInputStream(resultBuffer));
            }
        }
        else {
            try {
                bookList = getDocxBookMark(new ByteArrayInputStream(resultBuffer));
            }
            catch (final Exception e) {
                BookMarkUtil.logger.warn("获取docx书签出错", e);
                bookList = getDocBookMark(new ByteArrayInputStream(resultBuffer));
            }
        }
        if (BookMarkUtil.logger.isDebugEnabled()) {
            if (!ArrayUtil.isEmpty((List)bookList)) {
                BookMarkUtil.logger.debug("poi获取书签原结果：" + bookList.toString());
            }
            else {
                BookMarkUtil.logger.debug("poi获取书签原结果：无");
            }
        }
        final List<String> resultList = new ArrayList<String>();
        if (!ArrayUtil.isEmpty((List)bookList)) {
            for (final String book : bookList) {
                if (!book.startsWith("_")) {
                    resultList.add(book);
                }
            }
        }
        if (BookMarkUtil.logger.isDebugEnabled()) {
            if (!ArrayUtil.isEmpty((List)resultList)) {
                BookMarkUtil.logger.debug("poi获取书签结果：" + resultList.toString());
            }
            else {
                BookMarkUtil.logger.debug("poi获取书签结果：无");
            }
        }
        return resultList;
    }
    
    private static List<String> getDocBookMark(final InputStream in) throws Exception {
        List<String> bookList;
        try {
            bookList = new ArrayList<String>();
            final HWPFDocument doc = new HWPFDocument(in);
            final Bookmarks bookmarks = doc.getBookmarks();
            for (int i = 0, j = bookmarks.getBookmarksCount(); i < j; ++i) {
                final Bookmark bookmark = bookmarks.getBookmark(i);
                bookList.add(bookmark.getName());
            }
        }
        finally {
            IOUtils.closeQuietly(in);
        }
        IOUtils.closeQuietly(in);
        return bookList;
    }
    
    private static List<String> getDocxBookMark(final InputStream in) throws Exception {
        List<String> bookList;
        try {
            bookList = new ArrayList<String>();
            final XWPFDocument document = new XWPFDocument(in);
            final BookMarks bookMarks = new BookMarks(document);
            final Collection<BookMark> list = bookMarks.getBookmarkList();
            for (final BookMark bookMark : list) {
                bookList.add(bookMark.getBookmarkName());
            }
        }
        finally {
            IOUtils.closeQuietly(in);
        }
        IOUtils.closeQuietly(in);
        return bookList;
    }
    
    public static void main(final String[] args) throws Exception {
    }
}
